"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ApiClient = void 0;
const apiCall_1 = require("./API/apiCall");
const Auth_1 = require("./Auth");
const ApiCallOptions_1 = require("./Auth/ApiCallOptions");
const helpers_1 = require("./Auth/helpers");
const ConfigError_1 = require("./Errors/ConfigError");
const HttpStatusCodeError_1 = require("./Errors/HttpStatusCodeError");
const UserAPI_1 = require("./API/User/UserAPI");
const CategoryAPI_1 = require("./API/Category/CategoryAPI");
const ChannelAPI_1 = require("./API/Channel/ChannelAPI");
const MyselfAPI_1 = require("./API/User/MyselfAPI");
class ApiClient {
    /**
         * Creates a new API client instance.
         *
         * @param config Configuration for the client instance.
         */
    constructor(config) {
        if (!config.authProvider) {
            throw new ConfigError_1.ConfigError('No auth provider given. Please supply the `authProvider` option.');
        }
        this._config = config;
    }
    /**
     * Requests scopes from the auth provider.
     *
     * @param scopes The scopes to request.
     */
    async requestScopes(scopes) {
        await this._config.authProvider.getAccessToken(scopes);
    }
    /**
     * Retrieves information about your access token.
     */
    async getTokenInfo() {
        try {
            const data = await this.callApi({ method: "POST", type: ApiCallOptions_1.ApiCallType.Auth, url: 'validate' });
            return new Auth_1.TokenInfo(data);
        }
        catch (e) {
            if (e instanceof HttpStatusCodeError_1.HttpStatusCodeError && e.statusCode === 401) {
                throw new Auth_1.InvalidTokenError();
            }
            throw e;
        }
    }
    /**
     * Makes a call to the Glimesh API using your access token.
     *
     * @param options The configuration of the call.
     */
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    async callApi(options) {
        var _a;
        const { authProvider } = this._config;
        const shouldAuth = (_a = options.auth) !== null && _a !== void 0 ? _a : true;
        let accessToken = shouldAuth
            ? await authProvider.getAccessToken(options.scope ? [options.scope] : undefined)
            : null;
        if (!accessToken) {
            return apiCall_1.callApi(options, authProvider.clientId, undefined, this._config.fetchOptions);
        }
        if (accessToken.isExpired && authProvider.refresh) {
            const newAccessToken = await authProvider.refresh();
            if (newAccessToken) {
                accessToken = newAccessToken;
            }
        }
        let response = await this._callApiInternal(options, authProvider.clientId, accessToken.accessToken);
        if (response.status === 401 && authProvider.refresh) {
            await authProvider.refresh();
            accessToken = await authProvider.getAccessToken(options.scope ? [options.scope] : []);
            if (accessToken) {
                response = await this._callApiInternal(options, authProvider.clientId, accessToken.accessToken);
            }
        }
        return helpers_1.transformApiResponse(response);
    }
    /** @private */
    get _authProvider() {
        return this._config.authProvider;
    }
    async _callApiInternal(options, clientId, accessToken) {
        const { fetchOptions } = this._config;
        return apiCall_1.callApiRaw(options, clientId, accessToken, fetchOptions);
    }
    get categories() {
        return new CategoryAPI_1.CategoryAPI(this);
    }
    get channels() {
        return new ChannelAPI_1.ChannelAPI(this);
    }
    //TODO: Implement when available
    /*get followers(): FollowerAPI {
        return new FollowerAPI(this);
    }*/
    get myself() {
        if (this._config.authProvider instanceof Auth_1.ClientCredentialsAuthProvider) {
            throw new Auth_1.InvalidTokenTypeError("Access Token needed!");
        }
        return new MyselfAPI_1.MyselfAPI(this);
    }
    //TODO: Implement when available
    /*get subscriptions(): SubscriptionAPI {
        return new SubscriptionAPI(this);
    }*/
    get users() {
        return new UserAPI_1.UserAPI(this);
    }
    ;
}
exports.ApiClient = ApiClient;
