"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.RefreshableAuthProvider = void 0;
const __1 = require("..");
/**
 * Enhances another auth provider with the ability to make use of refresh
 * tokens, automatically refreshing the access token whenever necessary.
 */
class RefreshableAuthProvider {
    /**
     * Creates a new auth provider based on the given one that can automatically
     * refresh access tokens.
     *
     * @param childProvider The base auth provider.
     * @param refreshConfig The information necessary to automatically refresh an access token.
     */
    constructor(childProvider, refreshConfig) {
        this._clientSecret = refreshConfig.clientSecret;
        this._refreshToken = refreshConfig.refreshToken;
        this._childProvider = childProvider;
        this._initialExpiry = refreshConfig.expiry;
        this._onRefresh = refreshConfig.onRefresh;
    }
    /**
     * The type of tokens the provider generates.
     *
     * It is the same as the underlying base auth provider's token type.
     */
    get tokenType() {
        return this._childProvider.tokenType;
    }
    /**
     * Retrieves an access token.
     *
     * If the current access token does not have the requested scopes, the base auth
     * provider is called.
     *
     * If the current access token is expired, automatically renew it.
     *
     * @param scopes The requested scopes.
     */
    async getAccessToken(scopes) {
        if (typeof scopes === 'string') {
            scopes = scopes.split(' ');
        }
        const oldToken = await this._childProvider.getAccessToken();
        if (oldToken && scopes && scopes.some(scope => !this.currentScopes.includes(scope))) {
            // requesting a new scope should be delegated down...
            const newToken = await this._childProvider.getAccessToken(scopes);
            // ...but if the token doesn't change, carry on
            if (newToken !== oldToken) {
                return newToken;
            }
        }
        // if we don't have a current token, we just pass this and refresh right away
        if (oldToken) {
            if (this._initialExpiry) {
                const now = new Date();
                if (now < this._initialExpiry) {
                    return oldToken;
                }
            }
            else if (!oldToken.isExpired) {
                return oldToken;
            }
        }
        const refreshedToken = await this.refresh();
        if (oldToken) {
            return refreshedToken;
        }
        // need to check again for scopes after refreshing, in case a refresh token was passed without an access token
        return this._childProvider.getAccessToken(scopes);
    }
    /**
     * Force a refresh of the access token.
     */
    async refresh() {
        const tokenData = await __1.refreshUserToken(this.clientId, this._clientSecret, this._refreshToken);
        this.setAccessToken(tokenData);
        this._refreshToken = tokenData.refreshToken;
        this._initialExpiry = undefined;
        if (this._onRefresh) {
            this._onRefresh(tokenData);
        }
        return tokenData;
    }
    /** @private */
    setAccessToken(token) {
        this._childProvider.setAccessToken(token);
    }
    /**
     * The client ID.
     */
    get clientId() {
        return this._childProvider.clientId;
    }
    /**
     * The scopes that are currently available using the access token.
     */
    get currentScopes() {
        return this._childProvider.currentScopes;
    }
}
exports.RefreshableAuthProvider = RefreshableAuthProvider;
