"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.transformApiResponse = exports.getApiUrl = exports.getValidTokenFromProvider = exports.getTokenInfo = exports.revokeToken = exports.refreshUserToken = exports.getAppToken = exports.exchangeCode = void 0;
const _1 = require(".");
const apiCall_1 = require("../API/apiCall");
const HttpStatusCodeError_1 = require("../Errors/HttpStatusCodeError");
const ApiCallOptions_1 = require("./ApiCallOptions");
/**
 * Retrieves an access token with your client credentials and an authorization code.
 *
 * @param clientId The client ID of your application.
 * @param clientSecret The client secret of your application.
 * @param code The authorization code.
 * @param redirectUri The redirect URI. This serves no real purpose here, but must still match one of the redirect URIs you configured in the Glimesh Application dashboard.
 */
async function exchangeCode(clientId, clientSecret, code, redirectUri) {
    return new _1.AccessToken(await apiCall_1.callApi({
        type: ApiCallOptions_1.ApiCallType.Auth,
        url: 'token',
        method: 'POST',
        query: {
            grant_type: 'authorization_code',
            client_id: clientId,
            client_secret: clientSecret,
            code,
            redirect_uri: redirectUri
        }
    }));
}
exports.exchangeCode = exchangeCode;
/**
 * Retrieves an app access token with your client credentials.
 *
 * @param clientId The client ID of your application.
 * @param clientSecret The client secret of your application.
 * @param clientSecret
 */
async function getAppToken(clientId, clientSecret) {
    return new _1.AccessToken(await apiCall_1.callApi({
        type: ApiCallOptions_1.ApiCallType.Auth,
        url: 'token',
        method: 'POST',
        query: {
            grant_type: 'authorization_code',
            client_id: clientId,
            client_secret: clientSecret
        }
    }, clientId));
}
exports.getAppToken = getAppToken;
/**
 * Refreshes an expired access token with your client credentials and the refresh token that was given by the initial authentication.
 *
 * @param clientId The client ID of your application.
 * @param clientSecret The client secret of your application.
 * @param refreshToken The refresh token.
 */
async function refreshUserToken(clientId, clientSecret, refreshToken) {
    return new _1.AccessToken(await apiCall_1.callApi({
        type: ApiCallOptions_1.ApiCallType.Auth,
        url: 'token',
        method: 'POST',
        query: {
            grant_type: 'refresh_token',
            client_id: clientId,
            client_secret: clientSecret,
            refresh_token: refreshToken
        }
    }));
}
exports.refreshUserToken = refreshUserToken;
/**
 * Revokes an access token.
 *
 * @param clientId The client ID of your application.
 * @param accessToken The access token.
 */
async function revokeToken(clientId, accessToken) {
    await apiCall_1.callApi({
        type: ApiCallOptions_1.ApiCallType.Auth,
        url: 'revoke',
        method: 'POST',
        query: {
            client_id: clientId,
            token: accessToken
        }
    });
}
exports.revokeToken = revokeToken;
/**
 * Retrieves information about an access token.
 *
 * @param clientId The client ID of your application.
 * @param accessToken The access token to get the information of.
 */
async function getTokenInfo(accessToken, clientId) {
    try {
        const data = await apiCall_1.callApi({ type: ApiCallOptions_1.ApiCallType.Auth, url: 'validate' }, clientId, accessToken);
        return new _1.TokenInfo(data);
    }
    catch (e) {
        if (e instanceof HttpStatusCodeError_1.HttpStatusCodeError && e.statusCode === 401) {
            throw new _1.InvalidTokenError();
        }
        throw e;
    }
}
exports.getTokenInfo = getTokenInfo;
/** @private */
async function getValidTokenFromProvider(provider, scopes) {
    let lastTokenError = undefined;
    try {
        const accessToken = await provider.getAccessToken(scopes);
        if (accessToken) {
            // check validity
            const tokenInfo = await getTokenInfo(accessToken.accessToken);
            return { accessToken, tokenInfo };
        }
    }
    catch (e) {
        if (e instanceof _1.InvalidTokenError) {
            lastTokenError = e;
        }
    }
    if (provider.refresh) {
        try {
            const newToken = await provider.refresh();
            if (newToken) {
                // check validity
                const tokenInfo = await getTokenInfo(newToken.accessToken);
                return { accessToken: newToken, tokenInfo };
            }
        }
        catch (e) {
            if (e instanceof _1.InvalidTokenError) {
                lastTokenError = e;
            }
        }
    }
    throw lastTokenError !== null && lastTokenError !== void 0 ? lastTokenError : new Error('Could not retrieve a valid token');
}
exports.getValidTokenFromProvider = getValidTokenFromProvider;
/** @private */
function getApiUrl(url, type) {
    switch (type) {
        case ApiCallOptions_1.ApiCallType.Base:
            return `https://glimesh.tv/api`;
        case ApiCallOptions_1.ApiCallType.Auth:
            return `https://glimesh.tv/oauth/authorize`;
        case ApiCallOptions_1.ApiCallType.Custom:
            return url;
        default:
            return url;
    }
}
exports.getApiUrl = getApiUrl;
/** @private */
async function transformApiResponse(response) {
    if (!response.ok) {
        throw new HttpStatusCodeError_1.HttpStatusCodeError(response.status, response.statusText, await response.json());
    }
    if (response.status === 204) {
        return undefined; // oof
    }
    const text = await response.text();
    if (!text) {
        return undefined; // mega oof - glimesh doesn't return a response when it should
    }
    return JSON.parse(text);
}
exports.transformApiResponse = transformApiResponse;
